@@ -51,7 +51,7 @@ BackgroundEffectHelper::BackgroundEffectHelper(nux::View* view)
   TextureCache::GetDefault().themed_invalidated.connect(sigc::mem_fun(this, &BackgroundEffectHelper::LoadTextures));
   LoadTextures();
 
-  if (Settings::Instance().GetLowGfxMode())
+  if (Settings::Instance().low_gfx())
     blur_type = BLUR_NONE;
 }
 
@@ -147,6 +147,9 @@ bool BackgroundEffectHelper::UpdateOwnerGeometry()
 
 void BackgroundEffectHelper::UpdateBlurGeometries()
 {
+  if (blur_type == BLUR_NONE)
+      return;
+
   int radius = GetBlurRadius();
   blur_geometries_.clear();
   blur_geometries_.reserve(registered_list_.size());
