@@ -37,7 +37,7 @@ namespace decoration
 {
 namespace
 {
-const std::string MENUS_PANEL_NAME = "WindowLIM";
+const int SHADOW_BLUR_MARGIN_FACTOR = 2;
 }
 
 Window::Impl::Impl(Window* parent, CompWindow* win)
@@ -54,7 +54,6 @@ Window::Impl::Impl(Window* parent, CompWindow* win)
   , deco_elements_(cu::DecorationElement::NONE)
   , last_mwm_decor_(win_->mwmDecor())
   , last_actions_(win_->actions())
-  , panel_id_(MENUS_PANEL_NAME + std::to_string(win_->id()))
   , cv_(Settings::Instance().em())
 {
   active.changed.connect([this] (bool active) {
@@ -147,6 +146,23 @@ void Window::Impl::Undecorate()
   bg_textures_.clear();
 }
 
+void Window::Impl::UpdateWindowState(unsigned old_state)
+{
+  Update();
+
+  if (state_change_button_)
+  {
+    if (win_->state() & (CompWindowStateMaximizedVertMask|CompWindowStateMaximizedHorzMask))
+    {
+      state_change_button_->type = WindowButtonType::UNMAXIMIZE;
+    }
+    else
+    {
+      state_change_button_->type = WindowButtonType::MAXIMIZE;
+    }
+  }
+}
+
 void Window::Impl::UnsetExtents()
 {
   if (win_->hasUnmapReference())
@@ -409,7 +425,13 @@ void Window::Impl::SetupWindowControls()
     top_layout_->Append(std::make_shared<WindowButton>(win_, WindowButtonType::MINIMIZE));
 
   if (win_->actions() & (CompWindowActionMaximizeHorzMask|CompWindowActionMaximizeVertMask))
-    top_layout_->Append(std::make_shared<WindowButton>(win_, WindowButtonType::MAXIMIZE));
+  {
+    auto type = (win_->state() & (CompWindowStateMaximizedVertMask|CompWindowStateMaximizedHorzMask)) ?
+                 WindowButtonType::UNMAXIMIZE : WindowButtonType::MAXIMIZE;
+    auto state_change_button = std::make_shared<WindowButton>(win_, type);
+    top_layout_->Append(state_change_button);
+    state_change_button_ = state_change_button;
+  }
 
   auto title = std::make_shared<Title>();
   title->text = last_title_.empty() ? WindowManager::Default().GetWindowName(win_->id()) : last_title_;
@@ -492,6 +514,12 @@ bool Window::Impl::ShadowDecorated() const
   return deco_elements_ & cu::DecorationElement::SHADOW;
 }
 
+bool Window::Impl::ShapedShadowDecorated() const
+{
+  return deco_elements_ & cu::DecorationElement::SHADOW &&
+         deco_elements_ & cu::DecorationElement::SHAPED;
+}
+
 bool Window::Impl::FullyDecorated() const
 {
   return deco_elements_ & cu::DecorationElement::BORDER;
@@ -503,6 +531,14 @@ bool Window::Impl::ShouldBeDecorated() const
 }
 
 GLTexture* Window::Impl::ShadowTexture() const
+{
+  if (shaped_shadow_pixmap_)
+    return shaped_shadow_pixmap_->texture();
+
+  return SharedShadowTexture();
+}
+
+GLTexture* Window::Impl::SharedShadowTexture() const
 {
   auto const& mi = manager_->impl_;
   if (active() || parent_->scaled())
@@ -569,10 +605,22 @@ void Window::Impl::ComputeShadowQuads()
     if (!last_shadow_rect_.isEmpty())
       last_shadow_rect_.setGeometry(0, 0, 0, 0);
 
-    return;
+    shaped_shadow_pixmap_.reset();
   }
+  else if (deco_elements_ & cu::DecorationElement::SHAPED)
+  {
+    ComputeShapedShadowQuad();
+  }
+  else
+  {
+    shaped_shadow_pixmap_.reset();
+    ComputeGenericShadowQuads();
+  }
+}
 
-  const auto* texture = ShadowTexture();
+void Window::Impl::ComputeGenericShadowQuads()
+{
+  const auto* texture = SharedShadowTexture();
 
   if (!texture || !texture->width() || !texture->height())
     return;
@@ -676,6 +724,68 @@ void Window::Impl::ComputeShadowQuads()
   }
 }
 
+cu::PixmapTexture::Ptr Window::Impl::BuildShapedShadowTexture(nux::Size const& size, unsigned radius, nux::Color const& color, Shape const& shape) {
+  nux::CairoGraphics img(CAIRO_FORMAT_ARGB32, size.width, size.height);
+  auto* img_ctx = img.GetInternalContext();
+
+  for (auto const& rect : shape.GetRectangles())
+  {
+    cairo_rectangle(img_ctx, rect.x + radius * SHADOW_BLUR_MARGIN_FACTOR - shape.XOffset(), rect.y + radius * SHADOW_BLUR_MARGIN_FACTOR - shape.YOffset(), rect.width, rect.height);
+    cairo_set_source_rgba(img_ctx, color.red, color.green, color.blue, color.alpha);
+    cairo_fill(img_ctx);
+  }
+
+  img.BlurSurface(radius);
+
+  cu::CairoContext shadow_ctx(size.width, size.height);
+  cairo_set_source_surface(shadow_ctx, img.GetSurface(), 0, 0);
+  cairo_paint(shadow_ctx);
+
+  return shadow_ctx;
+}
+
+void Window::Impl::ComputeShapedShadowQuad()
+{
+  nux::Color color = active() ? manager_->active_shadow_color() : manager_->inactive_shadow_color();
+  unsigned int radius = active() ? manager_->active_shadow_radius() : manager_->inactive_shadow_radius();
+
+  Shape shape(win_->id());
+  auto const& border = win_->borderRect();
+  auto const& shadow_offset = manager_->shadow_offset();
+
+  // Ideally it would be shape.getWidth + radius * 2 but Cairographics::BlurSurface
+  // isn't bounded by the radius and we need to compensate by using a larger texture.
+  int width = shape.Width() + radius * 2 * SHADOW_BLUR_MARGIN_FACTOR;
+  int height = shape.Height() + radius * 2 * SHADOW_BLUR_MARGIN_FACTOR;
+
+  if (width != last_shadow_rect_.width() || height != last_shadow_rect_.height())
+    shaped_shadow_pixmap_ = BuildShapedShadowTexture({width, height}, radius, color, shape);
+
+  const auto* texture = shaped_shadow_pixmap_->texture();
+
+  if (!texture || !texture->width() || !texture->height())
+    return;
+
+  int x = border.x() + shadow_offset.x - radius * 2 + shape.XOffset();
+  int y = border.y() + shadow_offset.y - radius * 2 + shape.YOffset();
+
+  auto* quad = &shadow_quads_[Quads::Pos(0)];
+  quad->box.setGeometry(x, y, width, height);
+  quad->matrix = texture->matrix();
+  quad->matrix.x0 = -COMP_TEX_COORD_X(quad->matrix, quad->box.x1());
+  quad->matrix.y0 = -COMP_TEX_COORD_Y(quad->matrix, quad->box.y1());
+
+  CompRect shaped_shadow_rect(x, y, width, height);
+  if (shaped_shadow_rect != last_shadow_rect_)
+  {
+    auto const& win_region = win_->region();
+    quad->region = CompRegion(quad->box) - win_region;
+
+    last_shadow_rect_ = shaped_shadow_rect;
+    win_->updateWindowOutputExtents();
+  }
+}
+
 void Window::Impl::Paint(GLMatrix const& transformation,
                          GLWindowPaintAttrib const& attrib,
                          CompRegion const& region, unsigned mask)
@@ -714,14 +824,18 @@ void Window::Impl::Draw(GLMatrix const& transformation,
 
   glwin_->vertexBuffer()->begin();
 
-  for (unsigned i = 0; i < shadow_quads_.size(); ++i)
+  unsigned int num_quads = ShapedShadowDecorated() ? 1 : shadow_quads_.size();
+  for (unsigned int i = 0; i < num_quads; ++i)
   {
     auto& quad = shadow_quads_[Quads::Pos(i)];
     glwin_->glAddGeometry(quad.matrices, quad.region, clip_region);
   }
 
   if (glwin_->vertexBuffer()->end())
-    glwin_->glDrawTexture(ShadowTexture(), transformation, attrib, mask);
+  {
+    if (GLTexture* texture = ShadowTexture())
+      glwin_->glDrawTexture(texture, transformation, attrib, mask);
+  }
 
   for (auto const& dtex : bg_textures_)
   {
@@ -746,6 +860,9 @@ void Window::Impl::Damage()
 
 void Window::Impl::RedrawDecorations()
 {
+  if (!win_->isMapped())
+    return;
+
   dirty_geo_ = true;
   cwin_->damageOutputExtents();
 }
@@ -813,18 +930,13 @@ void Window::Impl::UpdateAppMenuVisibility()
     sliding_layout->mouse_owner = grab_edge_->mouse_owner();
 }
 
-inline std::string const& Window::Impl::GetMenusPanelID() const
-{
-  return panel_id_;
-}
-
 void Window::Impl::UnsetAppMenu()
 {
   if (!menus_)
     return;
 
   auto const& indicators = manager_->impl_->menu_manager_->Indicators();
-  indicators->SyncGeometries(GetMenusPanelID(), indicator::EntryLocationMap());
+  indicators->SyncGeometries(menus_->MenubarId(), indicator::EntryLocationMap());
   sliding_layout_->SetInputItem(nullptr);
   grab_mouse_changed_->disconnect();
 }
@@ -837,7 +949,7 @@ void Window::Impl::SyncMenusGeometries() const
   auto const& indicators = manager_->impl_->menu_manager_->Indicators();
   indicator::EntryLocationMap map;
   menus_->ChildrenGeometries(map);
-  indicators->SyncGeometries(GetMenusPanelID(), map);
+  indicators->SyncGeometries(menus_->MenubarId(), map);
 }
 
 bool Window::Impl::ActivateMenu(std::string const& entry_id)
@@ -907,6 +1019,11 @@ void Window::Update()
   impl_->Update();
 }
 
+void Window::UpdateWindowState(unsigned old_state)
+{
+  impl_->UpdateWindowState(old_state);
+}
+
 void Window::UpdateFrameRegion(CompRegion& r)
 {
   if (impl_->frame_region_.isEmpty())
