@@ -23,6 +23,7 @@
 #include <unordered_map>
 #include <NuxCore/NuxCore.h>
 #include <NuxCore/Rect.h>
+#include <NuxGraphics/CairoGraphics.h>
 #include <UnityCore/ConnectionManager.h>
 #include <UnityCore/Indicators.h>
 #include <core/core.h>
@@ -30,6 +31,7 @@
 #include <composite/composite.h>
 #include <X11/extensions/shape.h>
 
+#include "DecorationsShape.h"
 #include "DecorationsDataPool.h"
 #include "DecorationsManager.h"
 #include "DecorationsInputMixer.h"
@@ -47,6 +49,7 @@ class Title;
 class MenuLayout;
 class SlidingLayout;
 class ForceQuitDialog;
+class WindowButton;
 
 namespace cu = compiz_utils;
 
@@ -88,6 +91,7 @@ struct Window::Impl
   bool IsMaximized() const;
   bool FullyDecorated() const;
   bool ShadowDecorated() const;
+  bool ShapedShadowDecorated() const;
   void RedrawDecorations();
   void Damage();
   void SetupAppMenu();
@@ -100,6 +104,7 @@ private:
   void SetupExtents();
   void ComputeBorderExtent(CompWindowExtents &border);
   void UpdateElements(cu::WindowFilter wf = cu::WindowFilter::NONE);
+  void UpdateWindowState(unsigned old_state);
   void UpdateClientDecorationsState();
   void UpdateMonitor();
   void UpdateFrame();
@@ -117,14 +122,18 @@ private:
   void SyncMenusGeometries() const;
   bool ShouldBeDecorated() const;
   GLTexture* ShadowTexture() const;
+  GLTexture* SharedShadowTexture() const;
   unsigned ShadowRadius() const;
   std::string const& GetMenusPanelID() const;
 
   void ComputeShadowQuads();
+  void ComputeGenericShadowQuads();
+  void ComputeShapedShadowQuad();
   void UpdateDecorationTextures();
   void UpdateWindowEdgesGeo();
   void UpdateForceQuitDialogPosition();
   void RenderDecorationTexture(Side, nux::Geometry const&);
+  cu::PixmapTexture::Ptr BuildShapedShadowTexture(nux::Size const&, unsigned radius, nux::Color const&, Shape const&);
   void Paint(GLMatrix const&, GLWindowPaintAttrib const&, CompRegion const&, unsigned mask);
   void Draw(GLMatrix const&, GLWindowPaintAttrib const&, CompRegion const&, unsigned mask);
 
@@ -153,11 +162,12 @@ private:
   connection::Wrapper dpi_changed_;
   connection::Wrapper grab_mouse_changed_;
   std::string last_title_;
-  std::string panel_id_;
   std::vector<cu::SimpleTextureQuad> bg_textures_;
+  cu::PixmapTexture::Ptr shaped_shadow_pixmap_;
   std::shared_ptr<ForceQuitDialog> force_quit_;
   InputMixer::Ptr input_mixer_;
   Layout::Ptr top_layout_;
+  uweak_ptr<WindowButton> state_change_button_;
   uweak_ptr<MenuLayout> menus_;
   uweak_ptr<Title> title_;
   uweak_ptr<SlidingLayout> sliding_layout_;
