@@ -77,6 +77,8 @@ const std::string SUPPRESS_DIALOGS_KEY = "suppress-logout-restart-shutdown";
 
 const std::string GNOME_LOCKDOWN_OPTIONS = "org.gnome.desktop.lockdown";
 const std::string DISABLE_LOCKSCREEN_KEY = "disable-lock-screen";
+
+GDBusProxyFlags DEFAULT_CALL_FLAGS = static_cast<GDBusProxyFlags>(G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES|G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS|G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START_AT_CONSTRUCTION);
 }
 
 GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
@@ -99,37 +101,45 @@ GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
   });
 
   {
-    const char* session_id = test_mode_ ? "id0" : g_getenv("XDG_SESSION_ID");
+    std::string session_id = test_mode_ ? "id0" : glib::gchar_to_string(g_getenv("XDG_SESSION_ID"));
 
-    login_proxy_ = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.login1",
-                                                     "/org/freedesktop/login1/session/" + glib::gchar_to_string(session_id),
-                                                     "org.freedesktop.login1.Session",
-                                                     test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
-                                                     G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES);
+    if (!session_id.empty())
+    {
+      CallLogindMethod("GetSession", g_variant_new("(s)", session_id.c_str()), [this, session_id] (GVariant* variant, glib::Error const& err) {
+        std::string session_path;
 
-    login_proxy_->Connect("Lock", [this](GVariant*){
-      manager_->PromptLockScreen();
-    });
+        if (!err && variant)
+          session_path = glib::Variant(variant).GetObjectPath();
 
-    login_proxy_->Connect("Unlock", [this](GVariant*){
-      manager_->unlock_requested.emit();
-    });
+        if (session_path.empty())
+          session_path = "/org/freedesktop/login1/session/" + session_id;
 
-    login_proxy_->ConnectProperty("Active", [this] (GVariant* variant) {
-      bool active = glib::Variant(variant).GetBool();
-      manager_->screensaver_requested.emit(!active);
-      manager_->is_session_active.changed.emit(active);
-    });
+        SetupLogin1Proxy(session_path);
+      });
+    }
+    else
+    {
+      auto proxy = std::make_shared<glib::DBusProxy>("org.freedesktop.login1",
+                                                     "/org/freedesktop/login1/user/self",
+                                                     "org.freedesktop.login1.User",
+                                                     G_BUS_TYPE_SYSTEM);
 
-    manager_->is_session_active.SetGetterFunction([this] {
-      return login_proxy_->GetProperty("Active").GetBool();
-    });
+      proxy->GetProperty("Display", [this, proxy] (GVariant *variant) {
+        if (!variant || g_variant_n_children(variant) < 2)
+          return;
+
+        glib::Variant tmp(g_variant_get_child_value(variant, 1), glib::StealRef());
+        SetupLogin1Proxy(tmp.GetObjectPath());
+      });
+    }
   }
 
   {
     presence_proxy_ = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.gnome.SessionManager",
                                                         "/org/gnome/SessionManager/Presence",
-                                                        "org.gnome.SessionManager.Presence");
+                                                        "org.gnome.SessionManager.Presence",
+                                                        G_BUS_TYPE_SESSION,
+                                                        G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES);
 
     presence_proxy_->Connect("StatusChanged", [this](GVariant* variant) {
       enum class PresenceStatus : unsigned
@@ -165,7 +175,8 @@ GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
     dm_seat_proxy_ = std::make_shared<glib::DBusProxy>("org.freedesktop.Accounts",
                                                        ("/org/freedesktop/Accounts/User" + std::to_string(getuid())).c_str(),
                                                        "org.freedesktop.Accounts.User",
-                                                       G_BUS_TYPE_SYSTEM);
+                                                       G_BUS_TYPE_SYSTEM,
+                                                       G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS);
   }
 
   CallLogindMethod("CanHibernate", nullptr, [this] (GVariant* variant, glib::Error const& err) {
@@ -215,6 +226,34 @@ GnomeManager::Impl::~Impl()
   ClosedDialog();
 }
 
+void GnomeManager::Impl::SetupLogin1Proxy(std::string const& session_path)
+{
+  login_proxy_ = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.login1",
+                                                   session_path,
+                                                   "org.freedesktop.login1.Session",
+                                                   test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
+                                                   G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES);
+
+  login_proxy_->Connect("Lock", [this](GVariant*){
+    manager_->PromptLockScreen();
+  });
+
+  login_proxy_->Connect("Unlock", [this](GVariant*){
+    manager_->unlock_requested.emit();
+  });
+
+  login_proxy_->ConnectProperty("Active", [this] (GVariant* variant) {
+    bool active = glib::Variant(variant).GetBool();
+    manager_->is_session_active.changed.emit(active);
+    if (active)
+      manager_->screensaver_requested.emit(false);
+  });
+
+  manager_->is_session_active.SetGetterFunction([this] {
+    return login_proxy_->GetProperty("Active").GetBool();
+  });
+}
+
 bool GnomeManager::Impl::InteractiveMode()
 {
   bool schema_found = false;
@@ -373,7 +412,8 @@ void GnomeManager::Impl::CallGnomeSessionMethod(std::string const& method, GVari
                                                 glib::DBusProxy::CallFinishedCallback const& cb)
 {
   auto proxy = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.gnome.SessionManager",
-                                                 "/org/gnome/SessionManager", "org.gnome.SessionManager");
+                                                 "/org/gnome/SessionManager", "org.gnome.SessionManager",
+                                                 G_BUS_TYPE_SESSION, DEFAULT_CALL_FLAGS);
 
   // By passing the proxy to the lambda we ensure that it will be smartly handled
   proxy->CallBegin(method, parameters, [proxy, cb] (GVariant* ret, glib::Error const& e) {
@@ -391,7 +431,8 @@ void GnomeManager::Impl::CallUPowerMethod(std::string const& method, glib::DBusP
 {
   auto proxy = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.UPower",
                                                  "/org/freedesktop/UPower", "org.freedesktop.UPower",
-                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM);
+                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
+                                                 DEFAULT_CALL_FLAGS);
 
   proxy->CallBegin(method, nullptr, [proxy, cb] (GVariant *ret, glib::Error const& e) {
     if (e)
@@ -410,7 +451,8 @@ void GnomeManager::Impl::CallLogindMethod(std::string const& method, GVariant* p
   auto proxy = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.login1",
                                                  "/org/freedesktop/login1",
                                                  "org.freedesktop.login1.Manager",
-                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM);
+                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
+                                                 DEFAULT_CALL_FLAGS);
 
   // By passing the proxy to the lambda we ensure that it will be smartly handled
   proxy->CallBegin(method, parameters, [proxy, cb, method] (GVariant* ret, glib::Error const& e) {
@@ -431,7 +473,8 @@ void GnomeManager::Impl::CallConsoleKitMethod(std::string const& method, GVarian
   auto proxy = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.ConsoleKit",
                                                  "/org/freedesktop/ConsoleKit/Manager",
                                                  "org.freedesktop.ConsoleKit.Manager",
-                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM);
+                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
+                                                 DEFAULT_CALL_FLAGS);
 
   // By passing the proxy to the lambda we ensure that it will be smartly handled
   proxy->CallBegin(method, parameters, [this, proxy] (GVariant*, glib::Error const& e) {
@@ -449,7 +492,8 @@ void GnomeManager::Impl::CallDisplayManagerSeatMethod(std::string const& method,
   auto proxy = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.DisplayManager",
                                                  glib::gchar_to_string(xdg_seat_path),
                                                  "org.freedesktop.DisplayManager.Seat",
-                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM);
+                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
+                                                 DEFAULT_CALL_FLAGS);
   proxy->CallBegin(method, parameters, [this, proxy] (GVariant*, glib::Error const& e) {
     if (e)
     {
