@@ -50,6 +50,12 @@ const RawPixel VERTICAL_PADDING = 20_em;
 // Now that we mask the corners of the dash,
 // draw longer lines to fill the minimal gaps
 const RawPixel CORNER_OVERLAP = 3_em;
+
+enum class OverlayPosition
+{
+  LEFT,
+  BOTTOM
+};
 }
 
 // Impl class
@@ -82,7 +88,9 @@ public:
   nux::ObjectPtr<nux::BaseTexture> right_texture_;
   nux::ObjectPtr<nux::BaseTexture> right_texture_mask_;
   nux::ObjectPtr<nux::BaseTexture> left_texture_;
-  nux::ObjectPtr<nux::BaseTexture> top_bottom_texture_;
+  nux::ObjectPtr<nux::BaseTexture> top_left_texture_;
+  nux::ObjectPtr<nux::BaseTexture> top_texture_;
+  nux::ObjectPtr<nux::BaseTexture> bottom_texture_;
 
   nux::ObjectPtr<nux::BaseTexture> corner_;
   nux::ObjectPtr<nux::BaseTexture> corner_mask_;
@@ -119,8 +127,7 @@ OverlayRendererImpl::OverlayRendererImpl(OverlayRenderer *parent_)
   parent->scale = Settings::Instance().em()->DPIScale();
   parent->scale.changed.connect(sigc::hide(sigc::mem_fun(this, &OverlayRendererImpl::LoadScaledTextures)));
   parent->owner_type.changed.connect(sigc::hide(sigc::mem_fun(this, &OverlayRendererImpl::LoadScaledTextures)));
-  Settings::Instance().low_gfx_changed.connect(sigc::mem_fun(this, &OverlayRendererImpl::UpdateTextures));
-  Settings::Instance().launcher_position.changed.connect(sigc::hide(sigc::mem_fun(this, &OverlayRendererImpl::LoadScaledTextures)));
+  Settings::Instance().low_gfx.changed.connect(sigc::hide(sigc::mem_fun(this, &OverlayRendererImpl::UpdateTextures)));
   dash::Style::Instance().textures_changed.connect(sigc::mem_fun(this, &OverlayRendererImpl::UpdateTextures));
   dash::Style::Instance().textures_changed.connect(sigc::mem_fun(this, &OverlayRendererImpl::LoadScaledTextures));
 
@@ -132,24 +139,21 @@ void OverlayRendererImpl::LoadScaledTextures()
 {
   double scale = parent->scale;
   auto& style = dash::Style::Instance();
-  auto dash_position = dash::Position::LEFT;
 
-  if (Settings::Instance().launcher_position() == LauncherPosition::BOTTOM && parent->owner_type() == OverlayOwner::Dash)
-    dash_position = dash::Position::BOTTOM;
-
-  horizontal_texture_ = style.GetDashHorizontalTile(scale, dash_position);
-  horizontal_texture_mask_ = style.GetDashHorizontalTileMask(scale, dash_position);
+  horizontal_texture_ = style.GetDashHorizontalTile(scale);
+  horizontal_texture_mask_ = style.GetDashHorizontalTileMask(scale);
   right_texture_ = style.GetDashRightTile(scale);
   right_texture_mask_ = style.GetDashRightTileMask(scale);
+  top_left_texture_ = style.GetDashTopLeftTile(scale);
   left_texture_ = style.GetDashLeftTile(scale);
-  top_bottom_texture_ = style.GetDashTopOrBottomTile(scale, dash_position);
+  top_texture_ = style.GetDashTopTile(scale);
 
-  corner_ = style.GetDashCorner(scale, dash_position);
-  corner_mask_ = style.GetDashCornerMask(scale, dash_position);
-  left_corner_ = style.GetDashLeftCorner(scale, dash_position);
-  left_corner_mask_ = style.GetDashLeftCornerMask(scale, dash_position);
-  right_corner_ = style.GetDashRightCorner(scale, dash_position);
-  right_corner_mask_ = style.GetDashRightCornerMask(scale, dash_position);
+  corner_ = style.GetDashCorner(scale);
+  corner_mask_ = style.GetDashCornerMask(scale);
+  left_corner_ = style.GetDashLeftCorner(scale);
+  left_corner_mask_ = style.GetDashLeftCornerMask(scale);
+  right_corner_ = style.GetDashRightCorner(scale);
+  right_corner_mask_ = style.GetDashRightCornerMask(scale);
 }
 
 void OverlayRendererImpl::OnBgColorChanged(nux::Color const& new_color)
@@ -157,7 +161,7 @@ void OverlayRendererImpl::OnBgColorChanged(nux::Color const& new_color)
   bg_layer_->SetColor(new_color);
 
   //When we are in low gfx mode then our darken layer will act as a background.
-  if (Settings::Instance().GetLowGfxMode())
+  if (Settings::Instance().low_gfx())
   {
     bg_darken_layer_->SetColor(new_color);
   }
@@ -172,7 +176,7 @@ void OverlayRendererImpl::UpdateTextures()
   rop.SrcBlend = GL_ONE;
   rop.DstBlend = GL_ONE_MINUS_SRC_ALPHA;
 
-  if (Settings::Instance().GetLowGfxMode() || !nux::GetWindowThread()->GetGraphicsEngine().UsingGLSLCodePath())
+  if (Settings::Instance().low_gfx() || !nux::GetWindowThread()->GetGraphicsEngine().UsingGLSLCodePath())
   {
     auto& avg_color = WindowManager::Default().average_color;
     bg_layer_ = std::make_shared<nux::ColorLayer>(avg_color(), true, rop);
@@ -185,7 +189,7 @@ void OverlayRendererImpl::UpdateTextures()
   nux::Color darken_colour = nux::Color(0.9f, 0.9f, 0.9f, 1.0f);
 
   //When we are in low gfx mode then our darken layer will act as a background.
-  if (Settings::Instance().GetLowGfxMode())
+  if (Settings::Instance().low_gfx())
   {
     rop.Blend = false;
     rop.SrcBlend = GL_ONE;
@@ -565,7 +569,7 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
   bg_darken_layer_->SetGeometry(larger_content_geo);
   nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_darken_layer_.get());
 
-  if (!settings.GetLowGfxMode())
+  if (!settings.low_gfx())
   {
 #ifndef NUX_OPENGLES_20
     if (!gfx_context.UsingGLSLCodePath())
@@ -610,19 +614,19 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
 
   if (settings.form_factor() != FormFactor::NETBOOK || force_edges)
   {
-    int monitor = unity::UScreen::GetDefault()->GetMonitorWithMouse();
-    nux::Geometry const& monitor_geo = unity::UScreen::GetDefault()->GetMonitorGeometry(monitor);
+    int monitor = UScreen::GetDefault()->GetMonitorAtPosition(absolute_geo.x, absolute_geo.y);
+    auto const& monitor_geo = UScreen::GetDefault()->GetMonitorGeometry(monitor);
     int launcher_size = Settings::Instance().LauncherSize(monitor);
     int panel_height = panel::Style::Instance().PanelHeight(monitor);
 
-    auto dash_position = dash::Position::LEFT;
+    auto dash_position = OverlayPosition::LEFT;
     int border_y = content_geo.y;
     int border_height = larger_absolute_geo.height;
     if (parent->owner_type() == OverlayOwner::Dash && settings.launcher_position() == LauncherPosition::BOTTOM)
     {
       border_y = panel_height;
       border_height = monitor_geo.height - launcher_size;
-      dash_position = dash::Position::BOTTOM;
+      dash_position = OverlayPosition::BOTTOM;
     }
 
     nux::Geometry geo_border(content_geo.x, border_y, larger_absolute_geo.width - content_geo.x, border_height);
@@ -646,7 +650,13 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
       auto const& left_tile = left_texture_;
       auto const& right_corner = right_corner_;
       auto const& right_corner_mask = right_corner_mask_;
-      auto const& top_tile = top_bottom_texture_;
+      auto const& horizontal_tile = top_texture_;
+
+      if (dash_position == OverlayPosition::BOTTOM)
+      {
+        // Let's flip all the textures vertically
+        texxform.flip_v_coord = true;
+      }
 
       int left_corner_offset = LEFT_CORNER_OFFSET.CP(scale);
       int top_corner_offset = TOP_CORNER_OFFSET.CP(scale);
@@ -662,7 +672,7 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         texxform.SetWrap(nux::TEXWRAP_CLAMP_TO_BORDER, nux::TEXWRAP_CLAMP_TO_BORDER);
         int corner_y = geo.y + (geo.height - corner_size.height);
 
-        if (dash_position == dash::Position::BOTTOM)
+        if (dash_position == OverlayPosition::BOTTOM)
           corner_y = geo.y - corner_size.height + top_corner_offset;
 
         // Selectively erase blur region in the curbe
@@ -700,19 +710,18 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
       {
         // Horizontal repeated texture
         int real_width = geo.width - (left_corner_size.width - left_corner_offset) - corner_size.width;
-        int offset = real_width % horizontal->GetWidth();
         int horizontal_y = geo.y + (geo.height - horizontal->GetHeight());
 
-        if (dash_position == dash::Position::BOTTOM)
+        if (dash_position == OverlayPosition::BOTTOM)
           horizontal_y = geo.y - horizontal->GetHeight() + top_corner_offset;
 
         texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
         texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_REPEAT);
 
         // Selectively erase blur region in the curbe
-        gfx_context.QRP_ColorModTexAlpha(left_corner_size.width - left_corner_offset - offset,
+        gfx_context.QRP_ColorModTexAlpha(left_corner_size.width - left_corner_offset,
                                          horizontal_y,
-                                         real_width + offset,
+                                         real_width,
                                          horizontal->GetHeight(),
                                          horizontal_mask->GetDeviceTexture(),
                                          texxform,
@@ -721,9 +730,9 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         // Write correct alpha
         gfx_context.GetRenderStates().SetBlend(false);
         gfx_context.GetRenderStates().SetColorMask(false, false, false, true);
-        RenderInverseMask(gfx_context, left_corner_size.width - left_corner_offset - offset,
+        RenderInverseMask(gfx_context, left_corner_size.width - left_corner_offset,
                              horizontal_y,
-                             real_width + offset,
+                             real_width,
                              horizontal->GetHeight(),
                              horizontal_mask->GetDeviceTexture(),
                              texxform,
@@ -733,9 +742,9 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
         gfx_context.GetRenderStates().SetColorMask(true, true, true, true);
 
-        gfx_context.QRP_1Tex(left_corner_size.width - left_corner_offset - offset,
+        gfx_context.QRP_1Tex(left_corner_size.width - left_corner_offset,
                              horizontal_y,
-                             real_width + offset,
+                             real_width,
                              horizontal->GetHeight(),
                              horizontal->GetDeviceTexture(),
                              texxform,
@@ -747,10 +756,10 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         texxform.SetWrap(nux::TEXWRAP_CLAMP_TO_BORDER, nux::TEXWRAP_CLAMP_TO_BORDER);
         int left_corner_y = geo.y + (geo.height - left_corner_size.height);
 
-        if (dash_position == dash::Position::BOTTOM)
+        if (dash_position == OverlayPosition::BOTTOM)
           left_corner_y = geo.y - left_corner_size.height + top_corner_offset;
 
-        if (dash_position == dash::Position::LEFT)
+        if (dash_position == OverlayPosition::LEFT)
         {
           // Selectively erase blur region in the curbe
           gfx_context.QRP_ColorModTexAlpha(geo.x - left_corner_offset,
@@ -792,14 +801,14 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         int offset = real_height % left_tile->GetHeight();
         int left_texture_y = geo.y + geo.height;
 
-        if (dash_position == dash::Position::BOTTOM)
+        if (dash_position == OverlayPosition::BOTTOM)
         {
-          left_texture_y = panel_height;
-          real_height = monitor_geo.height - launcher_size - content_geo.height - left_corner->GetHeight() - panel_height + top_corner_offset;
+          left_texture_y = panel_height + top_left_texture_->GetHeight();
+          real_height = monitor_geo.height - launcher_size - content_geo.height - left_corner->GetHeight() - panel_height + top_corner_offset - top_left_texture_->GetHeight();
         }
         else if (settings.launcher_position() == LauncherPosition::BOTTOM)
         {
-          real_height -= launcher_size;
+          real_height -= launcher_size + top_left_texture_->GetWidth();
         }
 
         texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
@@ -819,7 +828,7 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_REPEAT);
         int right_edge_y = geo.y + right_corner_size.height - top_corner_offset;
 
-        if (dash_position == dash::Position::BOTTOM)
+        if (dash_position == OverlayPosition::BOTTOM)
           right_edge_y = geo.y + top_corner_offset;
 
         // Selectively erase blur region in the curbe
@@ -860,7 +869,7 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         texxform.SetWrap(nux::TEXWRAP_CLAMP_TO_BORDER, nux::TEXWRAP_CLAMP_TO_BORDER);
         int right_corner_y = geo.y - top_corner_offset;
 
-        if (dash_position == dash::Position::BOTTOM)
+        if (dash_position == OverlayPosition::BOTTOM)
           right_corner_y = geo.y + content_geo.height - right_corner_size.height + top_corner_offset;
 
         // Selectively erase blur region in the curbe
@@ -900,17 +909,79 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
         texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_REPEAT);
         int y = geo.y - top_corner_offset;
 
-        if (dash_position == dash::Position::BOTTOM)
-          y = geo.y + content_geo.height - top_tile->GetHeight() + top_corner_offset;
+        if (dash_position == OverlayPosition::BOTTOM)
+          y = geo.y + content_geo.height - horizontal_tile->GetHeight() + top_corner_offset;
 
         gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
         gfx_context.QRP_1Tex(geo.x + geo.width,
                              y,
                              geometry.width - (geo.x + geo.width),
-                             top_tile->GetHeight(),
-                             top_tile->GetDeviceTexture(),
+                             horizontal_tile->GetHeight(),
+                             horizontal_tile->GetDeviceTexture(),
                              texxform,
                              nux::color::White);
+
+        if (dash_position == OverlayPosition::BOTTOM)
+        {
+          // Top Left edge
+          nux::TexCoordXForm top_texxform;
+          top_texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
+          top_texxform.SetWrap(nux::TEXWRAP_CLAMP_TO_BORDER, nux::TEXWRAP_CLAMP_TO_BORDER);
+
+          gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
+          gfx_context.QRP_1Tex(0,
+                               panel_height,
+                               top_left_texture_->GetWidth(),
+                               top_left_texture_->GetHeight(),
+                               top_left_texture_->GetDeviceTexture(),
+                               top_texxform,
+                               nux::color::White);
+          // Top edge
+          top_texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
+          top_texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_REPEAT);
+          gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
+          gfx_context.QRP_1Tex(top_left_texture_->GetWidth(),
+                               panel_height - top_corner_offset,
+                               monitor_geo.width - top_left_texture_->GetWidth(),
+                               horizontal_tile->GetHeight(),
+                               horizontal_tile->GetDeviceTexture(),
+                               top_texxform,
+                               nux::color::White);
+        }
+        else if (settings.launcher_position() == LauncherPosition::BOTTOM)
+        {
+          int above_launcher_y = monitor_geo.height - panel_height - launcher_size;
+          bool was_flipped = texxform.flip_v_coord;
+
+          // Bottom Left edge
+          texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
+          texxform.SetWrap(nux::TEXWRAP_CLAMP_TO_BORDER, nux::TEXWRAP_CLAMP_TO_BORDER);
+          texxform.flip_v_coord = true;
+
+          gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
+          gfx_context.QRP_1Tex(0,
+                               above_launcher_y - top_left_texture_->GetWidth(),
+                               top_left_texture_->GetWidth(),
+                               top_left_texture_->GetHeight(),
+                               top_left_texture_->GetDeviceTexture(),
+                               texxform,
+                               nux::color::White);
+
+          // Bottom edge
+          texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
+          texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_REPEAT);
+
+          gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
+          gfx_context.QRP_1Tex(top_left_texture_->GetHeight(),
+                               above_launcher_y + top_corner_offset - horizontal_tile->GetHeight(),
+                               monitor_geo.width - top_left_texture_->GetHeight(),
+                               horizontal_tile->GetHeight(),
+                               horizontal_tile->GetDeviceTexture(),
+                               texxform,
+                               nux::color::White);
+
+          texxform.flip_v_coord = was_flipped;
+        }
       }
     }
 
@@ -984,7 +1055,7 @@ void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geo
   bgs++;
 
   //Only apply shine if we aren't in low gfx mode.
-  if (!Settings::Instance().GetLowGfxMode())
+  if (!Settings::Instance().low_gfx())
   {
 #ifndef NUX_OPENGLES_20
     if (!gfx_context.UsingGLSLCodePath())
