@@ -35,7 +35,6 @@
 #include "unity-shared/Introspectable.h"
 #include "unity-shared/MenuManager.h"
 #include "unity-shared/MockableBaseWindow.h"
-#include "unity-shared/Timer.h"
 #include "PanelMenuView.h"
 #include "PanelTray.h"
 #include "PanelIndicatorsView.h"
@@ -88,9 +87,6 @@ protected:
   void OnObjectAdded(indicator::Indicator::Ptr const& proxy);
   void OnObjectRemoved(indicator::Indicator::Ptr const& proxy);
   void OnIndicatorViewUpdated();
-  void OnMenuPointerMoved(int x, int y);
-  void OnEntryActivated(std::string const& panel, std::string const& entry_id, nux::Rect const& geo);
-  void OnEntryShowMenu(std::string const& entry_id, unsigned xid, int x, int y, unsigned button);
 
 private:
   std::string GetPanelName() const;
@@ -100,6 +96,8 @@ private:
   void OnSpreadInitiate();
   void OnSpreadTerminate();
   void OnLowGfxChanged();
+  void OnMenuPointerMoved(int x, int y, double speed);
+  void OnActiveEntryEvent(XEvent const&);
   void EnableOverlayMode(bool);
   void LoadTextures();
 
@@ -109,7 +107,6 @@ private:
   bool IsTransparent();
   void UpdateBackground();
   void ForceUpdateBackground();
-  bool TrackMenuPointer();
   void SyncGeometries();
   void AddPanelView(PanelIndicatorsView* child, unsigned int stretchFactor);
 
@@ -133,10 +130,6 @@ private:
   BaseTexturePtr bg_refine_single_column_tex_;
   std::unique_ptr<nux::AbstractPaintLayer> bg_refine_single_column_layer_;
 
-  std::string active_overlay_;
-  nux::Point  tracked_pointer_pos_, triangle_top_corner_;
-  util::Timer mouse_tracker_timer_;
-
   bool is_dirty_;
   bool opacity_maximized_toggle_;
   bool needs_geo_sync_;
@@ -144,15 +137,13 @@ private:
   float opacity_;
   int monitor_;
   int stored_dash_width_;
-
-  nux::Geometry menu_geo_;
+  std::string active_overlay_;
 
   connection::Manager on_indicator_updated_connections_;
   connection::Manager maximized_opacity_toggle_connections_;
   BackgroundEffectHelper bg_effect_helper_;
   nux::ObjectPtr<nux::IOpenGLBaseTexture> bg_blur_texture_;
   UBusManager ubus_manager_;
-  glib::Source::UniquePtr track_menu_pointer_timeout_;
 };
 
 } // namespace panel
