@@ -25,6 +25,7 @@
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/WindowManager.h"
 #include "unity-shared/IconRenderer.h"
+#include "unity-shared/UnitySettings.h"
 #include "unity-shared/UScreen.h"
 
 #include "SwitcherController.h"
@@ -44,7 +45,7 @@ const std::string SHOW_TIMEOUT = "show-timeout";
 const std::string DETAIL_TIMEOUT = "detail-timeout";
 const std::string VIEW_CONSTRUCT_IDLE = "view-construct-idle";
 const unsigned FADE_DURATION = 80;
-const int XY_OFFSET = 100;
+const RawPixel XY_OFFSET = 100_em;
 }
 
 namespace switcher
@@ -270,7 +271,7 @@ Controller::Impl::Impl(Controller* obj,
   ,  create_window_(create_window)
   ,  icon_renderer_(std::make_shared<ui::IconRenderer>())
   ,  main_layout_(nullptr)
-  ,  fade_animator_(FADE_DURATION)
+  ,  fade_animator_(Settings::Instance().low_gfx() ? 0 : FADE_DURATION)
 {
   WindowManager::Default().average_color.changed.connect(sigc::mem_fun(this, &Impl::OnBackgroundUpdate));
 
@@ -291,6 +292,10 @@ Controller::Impl::Impl(Controller* obj,
         HideWindow();
     }
   });
+
+  Settings::Instance().low_gfx.changed.connect(sigc::track_obj([this] (bool low_gfx) {
+    fade_animator_.SetDuration(low_gfx ? 0 : FADE_DURATION);
+  }, *this));
 }
 
 void Controller::Impl::OnBackgroundUpdate(nux::Color const& new_color)
@@ -436,7 +441,8 @@ nux::Geometry GetSwitcherViewsGeometry()
   int monitor      = uscreen->GetMonitorWithMouse();
   auto monitor_geo = uscreen->GetMonitorGeometry(monitor);
 
-  monitor_geo.Expand(-XY_OFFSET, -XY_OFFSET);
+  auto em = Settings::Instance().em(monitor);
+  monitor_geo.Expand(-XY_OFFSET.CP(em), -XY_OFFSET.CP(em));
 
   return monitor_geo;
 }
@@ -453,10 +459,6 @@ void Controller::Impl::ConstructView()
   view_->background_color = WindowManager::Default().average_color();
   view_->monitor = obj_->monitor_;
   view_->hide_request.connect(sigc::mem_fun(this, &Controller::Impl::Hide));
-  view_->switcher_mouse_up.connect([this] (int icon_index, int button) {
-    if (button == 3)
-      InitiateDetail(true);
-  });
 
   view_->switcher_mouse_move.connect([this] (int icon_index) {
     if (icon_index >= 0)
@@ -690,7 +692,10 @@ Selection Controller::Impl::GetCurrentSelection() const
       }
       else if (model_->SelectionIsActive())
       {
-        window = model_->SelectionWindows().front();
+        auto const& selection_windows = model_->SelectionWindows();
+
+        if (!selection_windows.empty())
+          window = selection_windows.front();
       }
     }
   }
