@@ -54,16 +54,16 @@ public:
   ~Settings();
 
   static Settings& Instance();
-  bool GetLowGfxMode() const;
-  void SetLowGfxMode(const bool low_gfx);
   EMConverter::Ptr const& em(int monitor = 0) const;
 
   void SetLauncherSize(int launcher_size, int monitor);
   int LauncherSize(int mointor) const;
 
+  nux::Property<bool> low_gfx;
   nux::RWProperty<FormFactor> form_factor;
   nux::Property<bool> is_standalone;
   nux::ROProperty<DesktopType> desktop_type;
+  nux::ROProperty<bool> pam_check_account_type;
   nux::ROProperty<bool> double_click_activate;
   nux::Property<unsigned> lim_movement_thresold;
   nux::Property<unsigned> lim_double_click_wait;
@@ -71,9 +71,13 @@ public:
   nux::Property<double> font_scaling;
   nux::ROProperty<bool> remote_content;
   nux::RWProperty<LauncherPosition> launcher_position;
+  nux::Property<bool> gestures_launcher_drag;
+  nux::Property<bool> gestures_dash_tap;
+  nux::Property<bool> gestures_windows_drag_pinch;
 
   sigc::signal<void> dpi_changed;
   sigc::signal<void> low_gfx_changed;
+  sigc::signal<void> gestures_changed;
 
 private:
   class Impl;
