@@ -20,10 +20,12 @@
 #include "config.h"
 
 #include "Nux/Nux.h"
+#include "Nux/NuxTimerTickSource.h"
 #include "Nux/VLayout.h"
 #include "Nux/WindowThread.h"
 #include "NuxGraphics/GraphicsEngine.h"
 #include <Nux/Layout.h>
+#include <NuxCore/AnimationController.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/Variant.h>
 #include <UnityCore/SocialPreview.h>
@@ -291,6 +293,9 @@ int main(int argc, char **argv)
                             &TestRunner::InitWindowThread,
                             test_runner);
 
+  nux::NuxTimerTickSource tick_source;
+  nux::animation::AnimationController animation_controller(tick_source);
+
   wt->Run (NULL);
   delete wt;
   return 0;
