@@ -70,6 +70,9 @@ R"(<node>
 const std::string LOGIND_MANAGER =
 R"(<node>
   <interface name="org.freedesktop.login1.Manager">
+    <method name="GetSession">
+      <arg type="s" name="result" direction="out"/>
+    </method>
     <method name="CanSuspend">
       <arg type="s" name="result" direction="out"/>
     </method>
@@ -186,7 +189,11 @@ struct TestGnomeSessionManager : testing::Test
     logind_->AddObjects(introspection::LOGIND_MANAGER, LOGIND_MANAGER_PATH);
     logind_->AddObjects(introspection::LOGIND_SESSION, LOGIND_SESSION_PATH);
     logind_->GetObjects().front()->SetMethodsCallsHandler([&] (std::string const& method, GVariant*) -> GVariant* {
-      if (method == "CanSuspend")
+      if (method == "GetSession")
+      {
+        return g_variant_new("(o)", "id0");
+      }
+      else if (method == "CanSuspend")
       {
         suspend_called = true;
         return g_variant_new("(s)", can_suspend_ ? "yes" : "no");
@@ -256,9 +263,19 @@ struct TestGnomeSessionManager : testing::Test
 
   void TearDown()
   {
+    manager->have_other_open_sessions.changed.clear();
+    manager->is_locked.changed.clear();
+    manager->is_session_active.changed.clear();
+    manager->lock_requested.clear();
+    manager->unlock_requested.clear();
+    manager->prompt_lock_requested.clear();
+    manager->locked.clear();
+    manager->unlocked.clear();
     manager->logout_requested.clear();
     manager->reboot_requested.clear();
     manager->shutdown_requested.clear();
+    manager->presence_status_changed.clear();
+    manager->screensaver_requested.clear();
     manager->cancel_requested.clear();
     shell_proxy_->DisconnectSignal();
 
@@ -493,7 +510,7 @@ TEST_F(TestGnomeSessionManager, Logout)
     if (method == "Logout")
     {
       logout_called = true;
-      EXPECT_EQ(Variant(par).GetUInt32(), 1);
+      EXPECT_EQ(Variant(par).GetUInt32(), 1u);
     }
 
     return nullptr;
