@@ -98,7 +98,7 @@ public:
   nux::ObjectPtr<nux::BaseWindow> parent_window_;
   dash::Preview::Ptr preview_model_;
 
-  previews::Style panel_style;
+  previews::Style previews_style;
   dash::Style dash_style;
   ThumbnailGenerator thumbnail_generator;
 };
@@ -122,7 +122,7 @@ TEST_F(TestPreviewApplication, TestUIValues)
   EXPECT_EQ(preview_view->copywrite_->GetText(), "Copywrite &amp; special char");
 
   EXPECT_EQ(preview_view->app_rating_->GetRating(), 0.8f);
-  EXPECT_EQ(preview_view->action_buttons_.size(), 2);
+  EXPECT_EQ(preview_view->action_buttons_.size(), 2u);
 
   if (preview_view->action_buttons_.size() >= 2)
   {
