@@ -213,12 +213,12 @@ TEST_F(TestVolumeLauncherIcon, WindowsOnMonitorChanges)
   ON_CALL(*file_manager_, WindowsForLocation(volume_->GetUri())).WillByDefault(Return(WindowList({win})));
   file_manager_->locations_changed.emit();
 
-  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(0), 1);
-  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(1), 0);
+  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(0), 1u);
+  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(1), 0u);
 
   win->SetMonitor(1);
-  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(0), 0);
-  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(1), 1);
+  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(0), 0u);
+  EXPECT_EQ(icon_->WindowsVisibleOnMonitor(1), 1u);
 }
 
 TEST_F(TestVolumeLauncherIcon, TestPosition)
@@ -481,7 +481,7 @@ TEST_F(TestVolumeLauncherIcon, TestEjectMenuItem)
   auto menuitem = GetMenuItemAtIndex(5);
 
   EXPECT_CALL(*volume_, Eject());
-  EXPECT_CALL(*notifications_, Display(volume_->GetIconName(), volume_->GetName()));
+  EXPECT_CALL(*notifications_, Display(volume_->GetName()));
 
   ASSERT_STREQ(dbusmenu_menuitem_property_get(menuitem, DBUSMENU_MENUITEM_PROP_LABEL), "Eject");
   EXPECT_TRUE(dbusmenu_menuitem_property_get_bool(menuitem, DBUSMENU_MENUITEM_PROP_VISIBLE));
@@ -573,7 +573,7 @@ TEST_F(TestVolumeLauncherIcon, TestEject)
     .WillRepeatedly(Return(true));
 
   EXPECT_CALL(*volume_, Eject());
-  EXPECT_CALL(*notifications_, Display(volume_->GetIconName(), volume_->GetName()));
+  EXPECT_CALL(*notifications_, Display(volume_->GetName()));
   icon_->EjectAndShowNotification();
 }
 
