@@ -2,7 +2,7 @@
 /* Compiz unity plugin
  * unity.h
  *
- * Copyright (c) 2010-11 Canonical Ltd.
+ * Copyright (c) 2010-16 Canonical Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,6 +56,7 @@
 #include "DashStyle.h"
 #include "EdgeBarrierController.h"
 #include "FavoriteStoreGSettings.h"
+#include "InputMonitor.h"
 #include "ShortcutController.h"
 #include "LauncherController.h"
 #include "LockScreenController.h"
@@ -69,12 +70,13 @@
 #include "SwitcherController.h"
 #include "SessionController.h"
 #include "SessionDBusManager.h"
-#include "SpreadFilter.h"
+#include "SpreadWidgets.h"
 #include "UBusWrapper.h"
 #include "UnityshellPrivate.h"
 #include "UnityShowdesktopHandler.h"
 #include "ThumbnailGenerator.h"
 #include "MenuManager.h"
+#include "UnityCore/SessionManager.h"
 
 #include "compizminimizedwindowhandler.h"
 #include "BGHash.h"
@@ -120,11 +122,58 @@ public:
   UnityScreen(CompScreen* s);
   ~UnityScreen();
 
-  /* We store these  to avoid unecessary calls to ::get */
-  CompScreen* screen;
-  CompositeScreen* cScreen;
-  GLScreen* gScreen;
-  ScaleScreen* sScreen;
+  switcher::Controller::Ptr switcher_controller();
+  launcher::Controller::Ptr launcher_controller();
+  lockscreen::Controller::Ptr lockscreen_controller();
+
+  void SetUpAndShowSwitcher(switcher::ShowMode show_mode = switcher::ShowMode::CURRENT_VIEWPORT);
+
+protected:
+  void damageCutoff() override;
+  void preparePaint(int ms) override;
+  void donePaint() override;
+
+  void handleCompizEvent(const char *pluginName, const char  *eventName, CompOption::Vector &o) override;
+  void damageRegion(const CompRegion &region) override;
+
+  /* paint on top of all windows if we could not find a window
+   * to paint underneath */
+  bool glPaintOutput(const GLScreenPaintAttrib&, const GLMatrix&, const CompRegion&, CompOutput*, unsigned int) override;
+
+  /* paint in the special case that the output is transformed */
+  void glPaintTransformedOutput(const GLScreenPaintAttrib&, const GLMatrix&, const CompRegion&, CompOutput*, unsigned int) override;
+
+  /* handle X11 events */
+  void handleEvent(XEvent*) override;
+  void addSupportedAtoms(std::vector<Atom>&) override;
+
+  /* handle showdesktop */
+  void enterShowDesktopMode() override;
+  void leaveShowDesktopMode(CompWindow *w) override;
+
+   /* window scaling */
+  bool layoutSlotsAndAssignWindows() override;
+  bool getMipmap() override { return false; }
+
+  /* Handle changes in the number of workspaces by showing the switcher
+   * or not showing the switcher */
+  bool setOptionForPlugin(const char* plugin, const char* name, CompOption::Value& v) override;
+
+  /* init plugin actions for screen */
+  bool initPluginForScreen(CompPlugin* p) override;
+
+  void outputChangeNotify() override;
+
+  CompAction::Vector& getActions() override;
+
+  std::string GetName() const override;
+  void AddProperties(debug::IntrospectionData&) override;
+
+private:
+  static void InitNuxThread(nux::NThread* thread, void* data);
+  void InitUnityComponents();
+  bool InitPluginActions();
+  void InitAltTabNextWindow();
 
   /* prepares nux for drawing */
   void nuxPrologue();
@@ -132,49 +181,10 @@ public:
   void nuxEpilogue();
 
   /* nux draw wrapper */
-  void paintDisplay();
+  void paintOutput();
   void paintPanelShadow(CompRegion const& clip);
   void setPanelShadowMatrix(const GLMatrix& matrix);
-
   void updateBlurDamage();
-  void damageCutoff();
-  void preparePaint (int ms);
-  void donePaint ();
-
-  void RaiseInputWindows();
-
-  void
-  handleCompizEvent (const char         *pluginName,
-                     const char         *eventName,
-                     CompOption::Vector &o);
-
-  void damageRegion(const CompRegion &region);
-
-  /* paint on top of all windows if we could not find a window
-   * to paint underneath */
-  bool glPaintOutput(const GLScreenPaintAttrib&,
-                     const GLMatrix&,
-                     const CompRegion&,
-                     CompOutput*,
-                     unsigned int);
-
-  /* paint in the special case that the output is transformed */
-  void glPaintTransformedOutput(const GLScreenPaintAttrib&,
-                                const GLMatrix&,
-                                const CompRegion&,
-                                CompOutput*,
-                                unsigned int);
-
-  /* handle X11 events */
-  void handleEvent(XEvent*);
-  void addSupportedAtoms(std::vector<Atom>&);
-
-  /* handle showdesktop */
-  void enterShowDesktopMode ();
-  void leaveShowDesktopMode (CompWindow *w);
-
-   /* window scaling */
-  bool layoutSlotsAndAssignWindows();
 
   bool showMenuBarInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool showMenuBarTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
@@ -190,9 +200,7 @@ public:
   bool setKeyboardFocusKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   bool altTabInitiateCommon(CompAction* action, switcher::ShowMode mode);
-  bool altTabTerminateCommon(CompAction* action,
-                             CompAction::State state,
-                             CompOption::Vector& options);
+  bool altTabTerminateCommon(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   bool altTabForwardInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool altTabPrevInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
@@ -201,10 +209,9 @@ public:
   bool altTabNextWindowInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool altTabPrevWindowInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
-  bool ShowHud();
-  /* handle hud key activations */
   bool ShowHudInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool ShowHudTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+
   bool launcherSwitcherForwardInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool launcherSwitcherPrevInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool launcherSwitcherTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
@@ -214,56 +221,29 @@ public:
   /* handle option changes and change settings inside of the
    * panel and dock views */
   void optionChanged(CompOption*, Options num);
+  void OnMinimizeDurationChanged();
 
-  /* Handle changes in the number of workspaces by showing the switcher
-   * or not showing the switcher */
-  bool setOptionForPlugin(const char* plugin, const char* name,
-                          CompOption::Value& v);
-
-  /* init plugin actions for screen */
-  bool initPluginForScreen(CompPlugin* p);
-
-  void outputChangeNotify();
   void NeedsRelayout();
   void ScheduleRelayout(guint timeout);
 
-  bool forcePaintOnTop ();
-
-  void SetUpAndShowSwitcher(switcher::ShowMode show_mode = switcher::ShowMode::CURRENT_VIEWPORT);
-
-  void OnMinimizeDurationChanged();
+  bool forcePaintOnTop();
 
   void OnLockScreenRequested();
   void OnScreenLocked();
   void OnScreenUnlocked();
   void SaveLockStamp(bool);
-
-  switcher::Controller::Ptr switcher_controller();
-  launcher::Controller::Ptr launcher_controller();
-  lockscreen::Controller::Ptr lockscreen_controller();
+  std::string GetLockStampFile() const;
 
   bool DoesPointIntersectUnityGeos(nux::Point const& pt);
 
   ui::LayoutWindow::Ptr GetSwitcherDetailLayoutWindow(Window window) const;
 
-  CompAction::Vector& getActions();
-
-protected:
-  std::string GetName() const;
-  void AddProperties(debug::IntrospectionData&);
-
-private:
   enum CancelActionTarget
   {
     LAUNCHER_SWITCHER,
     SHORTCUT_HINT
   };
 
-  static void InitNuxThread(nux::NThread* thread, void* data);
-  void InitUnityComponents();
-  bool InitPluginActions();
-  void InitAltTabNextWindow();
-
   void SendExecuteCommand();
 
   void EnsureSuperKeybindings();
@@ -273,21 +253,22 @@ private:
   void compizDamageNux(CompRegion const& region);
   void determineNuxDamage(CompRegion &nux_damage);
 
-  void onRedrawRequested();
   void Relayout();
+  void RaiseInputWindows();
 
   static void OnStartKeyNav(GVariant* data, void* value);
   static void OnExitKeyNav(GVariant* data, void* value);
 
   void restartLauncherKeyNav();
 
-  void OnDashRealized ();
-
+  bool ShowHud();
   void RaiseOSK();
 
+  void OnDashRealized();
   void OnLauncherStartKeyNav(GVariant* data);
   void OnLauncherEndKeyNav(GVariant* data);
   void OnSwitcherDetailChanged(bool detail);
+  void OnRedrawRequested();
 
   void OnInitiateSpread();
   void OnTerminateSpread();
@@ -304,23 +285,31 @@ private:
   void OnDecorationStyleChanged();
 
   void InitGesturesSupport();
+  void UpdateGesturesSupport();
 
   void DrawPanelUnderDash();
 
-  void FillShadowRectForOutput(CompRect &shadowRect,
-                               CompOutput const &output);
+  void FillShadowRectForOutput(CompRect &shadowRect, CompOutput const &output);
   unsigned CompizModifiersToNux(unsigned input) const;
   unsigned XModifiersToNux(unsigned input) const;
 
   void UpdateCloseWindowKey(CompAction::KeyBinding const&);
   void UpdateActivateIndicatorsKey();
 
-  bool getMipmap () override { return false; }
-
   void DamageBlurUpdateRegion(nux::Geometry const&);
 
   void ShowFirstRunHints();
   void SpreadAppWindows(bool anywhere);
+  bool queryForShader();
+
+  Window GetNextActiveWindow() const;
+  void SetNextActiveWindow(Window next_active_window);
+
+  /* We store these  to avoid unecessary calls to ::get */
+  CompScreen* screen;
+  CompositeScreen* cScreen;
+  GLScreen* gScreen;
+  ScaleScreen* sScreen;
 
   std::unique_ptr<na::TickSource> tick_source_;
   std::unique_ptr<na::AnimationController> animation_controller_;
@@ -331,6 +320,7 @@ private:
   internal::FavoriteStoreGSettings favorite_store_;
   ThumbnailGenerator thumbnail_generator_;
   lockscreen::Settings lockscreen_settings_;
+  input::Monitor input_monitor_;
 
   /* The window thread should be the last thing removed, as c++ does it in reverse order */
   std::unique_ptr<nux::WindowThread> wt;
@@ -354,7 +344,9 @@ private:
   ui::EdgeBarrierController::Ptr edge_barriers_;
   debug::DebugDBusInterface debugger_;
   std::unique_ptr<BGHash>   bghash_;
-  spread::Filter::Ptr       spread_filter_;
+  spread::Widgets::Ptr      spread_widgets_;
+
+  session::Manager::Ptr session_;
 
   /* Subscription for gestures that manipulate Unity launcher */
   std::unique_ptr<nux::GesturesSubscription> gestures_sub_launcher_;
@@ -365,8 +357,8 @@ private:
   /* Subscription for gestures that manipulate windows. */
   std::unique_ptr<nux::GesturesSubscription> gestures_sub_windows_;
 
-  bool                                  needsRelayout;
-  bool                                  super_keypressed_;
+  bool needsRelayout;
+  bool super_keypressed_;
   typedef std::shared_ptr<CompAction> CompActionPtr;
   typedef std::vector<CompActionPtr> ShortcutActions;
   ShortcutActions _shortcut_actions;
@@ -380,11 +372,10 @@ private:
   GLTexture::List _shadow_texture;
 
   /* handle paint order */
-  bool    doShellRepaint;
-  bool    didShellRepaint;
-  bool    allowWindowPaint;
-  bool    _key_nav_mode_requested;
-  CompOutput* _last_output;
+  bool doShellRepaint;
+  bool didShellRepaint;
+  bool allowWindowPaint;
+  CompOutput* last_output_;
 
   /* a small count-down work-a-around
    * to force full redraws of the shell
@@ -400,16 +391,14 @@ private:
 
   ::GLFramebufferObject *oldFbo;
 
-  bool   queryForShader ();
-
   int overlay_monitor_;
   CompScreen::GrabHandle grab_index_;
-  CompWindowList         fullscreen_windows_;
-  bool                   painting_tray_;
-  unsigned int           tray_paint_mask_;
-  unsigned int           last_scroll_event_;
-  int                    hud_keypress_time_;
-  int                    first_menu_keypress_time_;
+  CompWindowList fullscreen_windows_;
+  bool painting_tray_;
+  unsigned int tray_paint_mask_;
+  unsigned int last_scroll_event_;
+  int hud_keypress_time_;
+  int first_menu_keypress_time_;
 
   GLMatrix panel_shadow_matrix_;
 
@@ -418,9 +407,6 @@ private:
 
   bool scale_just_activated_;
   WindowMinimizeSpeedController minimize_speed_controller_;
-
-  uint64_t big_tick_;
-
   debug::ScreenIntrospection screen_introspection_;
 
   UBusManager ubus_manager_;
@@ -430,12 +416,14 @@ private:
 
   CompRegion buffered_compiz_damage_this_frame_;
   CompRegion buffered_compiz_damage_last_frame_;
-  bool       ignore_redraw_request_;
-  bool       dirty_helpers_on_this_frame_;
-
+  bool ignore_redraw_request_;
+  bool dirty_helpers_on_this_frame_;
+  bool is_desktop_active_;
+  bool key_nav_mode_requested_;
+  uint64_t big_tick_;
   unsigned int back_buffer_age_;
 
-  bool is_desktop_active_;
+  Window next_active_window_;
 
   friend class UnityWindow;
   friend class debug::ScreenIntrospection;
@@ -445,6 +433,7 @@ private:
 class UnityWindow :
   public WindowInterface,
   public GLWindowInterface,
+  public CompositeWindowInterface,
   public ShowdesktopHandlerWindowInterface,
   public compiz::WindowInputRemoverLockAcquireInterface,
   public WrapableHandler<ScaleWindowInterface, 4>,
@@ -457,98 +446,91 @@ public:
   UnityWindow(CompWindow*);
   ~UnityWindow();
 
-  CompWindow* window;
-  CompositeWindow* cWindow;
-  GLWindow* gWindow;
+  void minimize() override;
+  void unminimize() override;
+  bool minimized() const override;
+  bool focus() override;
+  void activate() override;
 
-  nux::Geometry last_bound;
+  //! Emited when CompWindowNotifyBeforeDestroy is received
+  sigc::signal<void> being_destroyed;
 
-  void minimize();
-  void unminimize();
-  bool minimized() const;
-  bool focus();
-  void activate();
-
-  void updateFrameRegion(CompRegion &region);
-  void getOutputExtents(CompWindowExtents& output);
+protected:
+  void updateFrameRegion(CompRegion &region) override;
+  void getOutputExtents(CompWindowExtents& output) override;
 
   /* occlusion detection
    * and window hiding */
-  bool glPaint(GLWindowPaintAttrib const&, GLMatrix const&, CompRegion const&, unsigned mask);
+  bool glPaint(GLWindowPaintAttrib const&, GLMatrix const&, CompRegion const&, unsigned mask) override;
 
   /* basic window draw function */
-  bool glDraw(GLMatrix const&, GLWindowPaintAttrib const&, CompRegion const&, unsigned mask);
+  bool glDraw(GLMatrix const&, GLWindowPaintAttrib const&, CompRegion const&, unsigned mask) override;
+  bool damageRect(bool initial, CompRect const&) override;
 
-  bool damageRect(bool initial, CompRect const&);
+  void updateIconPos(int &wx, int &wy, int x, int y, float width, float height) override;
+  void windowNotify(CompWindowNotify n) override;
+  void moveNotify(int x, int y, bool immediate) override;
+  void resizeNotify(int x, int y, int w, int h) override;
+  void stateChangeNotify(unsigned int lastState) override;
 
-  void updateIconPos (int &wx, int &wy, int x, int y, float width, float height);
-  void windowNotify(CompWindowNotify n);
-  void moveNotify(int x, int y, bool immediate);
-  void resizeNotify(int x, int y, int w, int h);
-  void stateChangeNotify(unsigned int lastState);
+  bool place(CompPoint& pos) override;
+  void scalePaintDecoration(GLWindowPaintAttrib const&, GLMatrix const&, CompRegion const&, unsigned mask) override;
+
+  std::string GetName() const override;
+  void AddProperties(debug::IntrospectionData&) override;
+
+private:
+  typedef compiz::CompizMinimizedWindowHandler<UnityScreen, UnityWindow> UnityMinimizedHandler;
+  typedef std::shared_ptr<compiz_utils::PixmapTexture> PixmapTexturePtr;
+
+  void DoEnableFocus();
+  void DoDisableFocus();
+
+  bool IsOverrideRedirect();
+  bool IsManaged();
+  bool IsGrabbed();
+  bool IsDesktopOrDock();
+  bool IsSkipTaskbarOrPager();
+  bool IsHidden();
+  bool IsInShowdesktopMode();
+  bool IsShaded();
+  bool IsMinimized();
+  bool CanBypassLockScreen() const;
+  void DoOverrideFrameRegion(CompRegion &r);
+
+  void DoHide();
+  void DoNotifyHidden();
+  void DoShow();
+  void DoNotifyShown();
+
+  void OnInitiateSpread();
+  void OnTerminateSpread();
 
-  bool place(CompPoint& pos);
   CompPoint tryNotIntersectUI(CompPoint& pos);
   nux::Geometry GetScaledGeometry();
   nux::Geometry GetLayoutWindowGeometry();
 
-  void paintThumbnail(nux::Geometry const& bounding, float parent_alpha, float alpha, float scale_ratio, unsigned deco_height, bool selected);
-
   void enterShowDesktop();
   void leaveShowDesktop();
   bool HandleAnimations(unsigned int ms);
 
   bool handleEvent(XEvent *event);
-  void scalePaintDecoration(GLWindowPaintAttrib const&, GLMatrix const&, CompRegion const&, unsigned mask);
+  void paintThumbnail(nux::Geometry const& bounding, float parent_alpha, float alpha, float scale_ratio, unsigned deco_height, bool selected);
 
-  //! Emited when CompWindowNotifyBeforeDestroy is received
-  sigc::signal<void> being_destroyed;
+  void DoAddDamage();
+  ShowdesktopHandlerWindowInterface::PostPaintAction DoHandleAnimations(unsigned int ms);
 
+  void DoMoveFocusAway();
 
-protected:
-  std::string GetName() const;
-  void AddProperties(debug::IntrospectionData&);
+  void DoDeleteHandler();
 
-private:
-  typedef compiz::CompizMinimizedWindowHandler<UnityScreen, UnityWindow> UnityMinimizedHandler;
-  typedef std::shared_ptr<compiz_utils::PixmapTexture> PixmapTexturePtr;
+  unsigned int GetNoCoreInstanceMask();
 
-  void DoEnableFocus ();
-  void DoDisableFocus ();
-
-  bool IsOverrideRedirect ();
-  bool IsManaged ();
-  bool IsGrabbed ();
-  bool IsDesktopOrDock ();
-  bool IsSkipTaskbarOrPager ();
-  bool IsHidden ();
-  bool IsInShowdesktopMode ();
-  bool IsShaded ();
-  bool IsMinimized ();
-  bool CanBypassLockScreen() const;
-  void DoOverrideFrameRegion (CompRegion &r);
-
-  void DoHide ();
-  void DoNotifyHidden ();
-  void DoShow ();
-  void DoNotifyShown ();
-
-  void OnInitiateSpread();
-  void OnTerminateSpread();
-
-  void DoAddDamage ();
-  ShowdesktopHandlerWindowInterface::PostPaintAction DoHandleAnimations (unsigned int ms);
-
-  void DoMoveFocusAway ();
-
-  void DoDeleteHandler ();
-
-  unsigned int GetNoCoreInstanceMask ();
-
-  compiz::WindowInputRemoverLock::Ptr GetInputRemover ();
+  compiz::WindowInputRemoverLock::Ptr GetInputRemover();
 
   void RenderDecoration(compiz_utils::CairoContext const&, double aspect = 1.0f);
-  void RenderTitle(compiz_utils::CairoContext const&, int x, int y, int width, int height, double aspect = 1.0f);
+  void RenderTitle(compiz_utils::CairoContext const&, int x, int y, int width,
+                   int height, double aspect = 1.0f);
   void DrawTexture(GLTexture::List const& textures, GLWindowPaintAttrib const&,
                    GLMatrix const&, unsigned mask, int x, int y, double aspect = 1.0f);
 
@@ -565,7 +547,12 @@ public:
   std::unique_ptr <UnityMinimizedHandler> mMinimizeHandler;
 
 private:
+  CompWindow* window;
+  CompositeWindow* cWindow;
+  GLWindow* gWindow;
+
   std::unique_ptr <ShowdesktopHandler> mShowdesktopHandler;
+  nux::Geometry last_bound;
   PixmapTexturePtr decoration_tex_;
   PixmapTexturePtr decoration_selected_tex_;
   std::string decoration_title_;
@@ -579,6 +566,7 @@ private:
   glib::Source::UniquePtr focus_desktop_timeout_;
 
   friend class UnityScreen;
+  friend UnityMinimizedHandler;
 };
 
 
